package java.awt;

import java.util.EmptyStackException;
import java.util.Vector;
import java.util.Enumeration;
import java.lang.reflect.InvocationTargetException;
import java.awt.event.*;

/*
  The Java APIs suck. I have no idea about how the queue push/pop mechanism
  is supposed to work. Here are some guesses...

  ,--- push new queues here
  |
  |    ,--- next event
  |    |      
  |    |      ,--- new events are posted here
  |    |      |
  V    V      V
     +---------+ next     prev +--------------+
  |--| 1 2 3   |---------------|              |-- ... --|
     +---------+               +--------------+

     requests are forwarded to next as long as next!=null.

*/
public class EventQueue {

    EventQueue next;
    EventQueue previous;

    /* FIXME: using Vector here is inefficient. LinkedList should be used */
    Vector pendingEvents = new Vector();

    EventQueue() {}

    public synchronized void postEvent(AWTEvent newEvent) {
	if (next != null) {
	    // ...flush own events?...
	    next.postEvent(newEvent);
	    notifyAll();
	    return;
	}

	Object src = newEvent.getSource();
	int id = newEvent.getID();

	/* It is not enough to use peekEvent() to find the first event
           of a perticular type, since for coalescing to be effective,
           all matching events must be tested. */

	boolean wasEmpty = (pendingEvents.size() == 0);
	
	int pos = -1;
    coalesce:
	while (true) {
	    int count = pendingEvents.size();
	    for (int i=count-1; i>=0; i--) {
		AWTEvent queued = (AWTEvent) pendingEvents.elementAt(i);
		if (queued.getSource() != src) continue;
		if (queued.getID() != id) continue;
		AWTEvent combined =
		    ((Component) src).coalesceEvents(queued, newEvent);
		if (combined != null) {
		    pendingEvents.removeElementAt(i);
		    pos = i;
		    newEvent = combined;
		    continue coalesce;
		}
	    }
	    break;
	}

	/* Insert the new event at the position of the old, so that
	   events are not delayed indefinitely by a steady stream of
	   new events. */
	if (pos != -1) {
	    pendingEvents.insertElementAt(newEvent, pos);
	} else {
	    pendingEvents.addElement(newEvent);
	}
	if (wasEmpty) notifyAll();
    }

    public synchronized AWTEvent getNextEvent()
	throws InterruptedException {
	if (next != null) return next.getNextEvent();

	while (pendingEvents.size() == 0) {
	    wait();
	}
	AWTEvent event = (AWTEvent) pendingEvents.elementAt(0);
	pendingEvents.removeElementAt(0);
	return event;
    }

    public synchronized AWTEvent peekEvent() {
	if (next != null) return next.peekEvent();
	if (pendingEvents.size() == 0) return null;
	return (AWTEvent) pendingEvents.elementAt(0);
    }

    public synchronized AWTEvent peekEvent(int id) {
	if (next != null) return next.peekEvent(id);

	Enumeration enum = pendingEvents.elements();
	while (enum.hasMoreElements()) {
	    AWTEvent event = (AWTEvent) enum.nextElement();
	    if (event.getID() == id) return event;
	}
	
	return null;
    }

    protected void dispatchEvent(AWTEvent newEvent) {
	Object source = newEvent.getSource();
	if (source instanceof Component) {
	    Component component = (Component) source;
	    component.dispatchEvent(newEvent);
	    return;
	}

	/*
	if (source instanceof MenuComponent) {
	    MenuComponent menuComponent = (MenuComponent) source;
	    menuComponent.dispatchEvent(newEvent);
	    return;
	}
	*/


	/* FIXME: What the hell is an ActiveEvent?

	if (newEvent instanceof ActiveEvent) {
	    ActiveEvent activeEvent = (ActiveEvent) newEvent;
	    activeEvent.dispatch();
	    
	}
	*/

	// Ignore unrecognized events...
	return;
    }

    private synchronized AWTEvent nonblockingGetNextEvent() {
	if (peekEvent() == null) return null;
	try {
	    return getNextEvent();
	} catch (InterruptedException ie) {
	    // FIXME: what now?
	    throw new RuntimeException(ie.toString());
	}
    }

    public synchronized void push(EventQueue newEventQueue) {
	if (next != null) {
	    next.push(newEventQueue);
	    return;
	}
	
	// drain this event queue into the next.
	while (peekEvent() != null) {
	    newEventQueue.postEvent(nonblockingGetNextEvent());
	}
	
	// couple queues
	next = newEventQueue;
	newEventQueue.previous = this;
    }

    protected synchronized void pop() throws EmptyStackException {
	if (previous == null) {
	    throw new EmptyStackException();
	}

	// FIXME: some locking is needed here...

	// decouple queues
	EventQueue prev = previous;
	previous.next = null;
	previous = null;
	
	// push back events this event queue into the previous queue.
	while (peekEvent() != null) {
	    prev.postEvent(nonblockingGetNextEvent());
	}
    }

    public static boolean isDispatchThread() {
	// should perhaps be...
	//Toolkit.getDefaultToolkit().getSystemEventQueue().isDispatchThreadImpl();

	throw new UnsupportedOperationException("not implemented yet");
    }
    
    private boolean isDispatchThreadImpl() {
	if (next != null) return next.isDispatchThread();
	throw new UnsupportedOperationException("not implemented yet");
    }
    


    public static void invokeLater(Runnable runnable) {
	throw new UnsupportedOperationException("not implemented yet");
	
    }
    

    public static void invokeAndWait(Runnable runnable)
	throws InterruptedException, InvocationTargetException {
	throw new UnsupportedOperationException("not implemented yet");
    }
    

}
