package java.awt;

import java.io.Serializable;

public abstract class FontMetrics implements Serializable {
    protected Font font;

    protected FontMetrics(Font font) {
	this.font = font;
    }

    public Font getFont() {
	return font;
    }

    public int getLeading() {
	return getMaxAscent() + getMaxDescent() -
	    getAscent() - getDescent();
    }

    public int getAscent() {
	return getHeight() - getLeading() - getDescent();
    }
    
    public int getDescent() {
	return getHeight() - getLeading() - getAscent();
    }

    public int getHeight() {
	return getLeading() + getAscent() + getDescent();
    }

    public int getMaxAscent() {
	return getAscent();
    }

    public int getMaxDescent() {
	return getDescent();
    }
    
    // I won't implement misspelling
    //public int getMaxDecent();

    public int getMaxAdvance() {
	return -1;
    }

    public int charWidth(int ch) {
	return charWidth((char) ch);
    }

    public int charWidth(char ch) {
	Character chObj = new Character(ch);
	return stringWidth(chObj.toString());
    }
    
    public int stringWidth(String str) {
	char[] chars = str.toCharArray();
	return charsWidth(chars, 0, chars.length);
    }

    public int charsWidth(char[] data, int off, int len) {
	return stringWidth(new String(data, off, len));
    }

    public int bytesWidth(byte[] data, int off, int len) {
	return stringWidth(new String(data, off, len));
    }
    
    public int[] getWidths() {
	int[] widths = new int[256];
	for (char c=0; c<256; c++) widths[c] = charWidth(c);
	return widths;
    }

    public boolean hasUniformLineMetrics() {
	return true;
    }

    /*
    public LineMetrics getLineMetrics(String str, Graphics context);

    public LineMetrics getLineMetrics(String str, int beginIndex,
				      int limit,
				      Graphics context);

    public LineMetrics getLineMetrics(char[] chars, int beginIndex,
				      int limit,
				      Graphics context);

    public LineMetrics getLineMetrics(CharacterIterator ci, int beginIndex,
				      int limit,
				      Graphics context);

    public Rectangle2D getStringBounds(String str, Graphics context);

    public Rectangle2D getStringBounds(String str, int beginIndex,
				       int limit, Graphics context);

    public Rectangle2D getStringBounds(char[] chars, int beginIndex,
				       int limit, Graphics context);

    public Rectangle2D getStringBounds(CharacterIterator ci, int beginIndex,
				       int limit, Graphics context);

    public Rectangle2D getMaxCharBounds(Graphics context);
    */

    //public String toString();
    
}
