package java.awt;
//import java.util.Set;
//import java.util.TreeSet;
import java.util.Locale;
import java.awt.peer.*;
import java.awt.event.*;

/* Most commented out code is due to lacking implementation in other
   classes. 



   Notice that the semantics of the parent property has changed in
   this subclass. The parent - child relationship here does not imply
   actual embedding of components, but simply logical grouping of
   toplevel windows..  */
public class Window extends Container {

    private Locale locale;
    //private Set owned;

    // convenience method
    private final WindowPeer peer() {
	return (WindowPeer) peer;
    }


    /** This (package access) constructor is used by subclasses that
	want to build windows that do not have parents. Eg. toplevel
	application frames. */
    Window() {
	setVisible(false);
	setLayout(new BorderLayout());
    }
    
    
    public Window(Frame owner) {
	this((Window) owner);
    }

    public Window(Window owner) {
	this();
	if (owner == null)
	    throw new IllegalArgumentException("owner must not be null");
	
	this.parent = owner;
	//owner.owned.add(this); // this should be a weak reference
    }

    /*
    public Window(Window owner, GraphicsConfiguration gc); 
    */

    protected void finalize() throws Throwable {
	//owner.owned.remove(this); 
	/* Won't work since finalize will never be called as long
	   as owned contains a reference to the object. */
	
	super.finalize();
    }


    public void addNotify() {
	if (peer == null) {
	    // why is cast needed?
	    // FIXME: implement createWindow
	    //peer = (ComponentPeer) getToolkit().createWindow(this);
	}
	super.addNotify();
    }
    
    public void pack() {

	/* Efficiency problem:

	   We've got a catch-22 here. An XFramePeer wants to know its
	   dimensions before creating an X window. However, the
	   Javadocs for this method states that the peer is created
	   before querying for preferred size, so that the peer gets
	   to decide the size. X windows does not have any perticular
	   preference as to size since the border decoration is not
	   included in the size measurements. An X window would
	   therefore rather be told what size to set during creation,
	   skipping an extra roundtrip to set bounds later.

	   This can perhaps be solved by inserting some code here that
	   probes the toolkit to find out whether it wants to create
	   the peer after setting preferred size rather than before.  */

	/* On second thought, it may be better to just to create the
           peer and set the real bounds later. The peer implementation
           can delay the actual creation of native resources (eg. an X
           window) until the component is made visible for the first
           time. */


	/* The current approach is going to hurt us if getPreferredSize()
	   is costly. */

	


	// new strategy:

	// set size based on current size estimate (without peers)
	Dimension size = getPreferredSize();
	setSize(getPreferredSize());

	/* Then validate (i.e. lay out components) without peers,
           assuming peers that has a size preference different than
           the component frontend, will call invalidate() to ensure
           that layout is redone. */
	validate(); 


	if (peer == null) {
	    addNotify();

	    // The peer might have a different idea about the preferred
	    // size. 
	    size = getPreferredSize();
	    if (!getSize().equals(size)) setSize(size);    
	}
	
	// revalidate sizes based on contributions from peers (if any).
	validate();
    }
    
    public void show() {
	/* I would rather do this in getXXXSize, but hey... */
	if (peer == null) addNotify();
	validate();
	
	/* I can't imagine this being useful behavior for an application,
	   but this is what the Javadocs says... */
	//if ((parent != null) && (!parent.isVisible())) parent.show();
	if (/* isVisible() /**/ true /**/) {
	    toFront();
	} else {
	    setVisible(true);
	}
    }

    public void hide() {
	setVisible(false);
    }

    public void dispose() {
	Window[] list = getOwnedWindows();
	for (int i=0; i<list.length; i++) {
	    list[i].dispose();
	}
	hide();
	//removeNotify();
    }
    

    public void toFront() {
	//peer().toFront();
    }

    public void toBack() {
	//peer().toBack();
    }

    /* Implementation according to Component API
    Toolkit getToolkit() {
	boolean lightweight = isLightweight(); 
	return Toolkit.getNativeContainer(this).peer.getToolkit();
    }*/

    public Toolkit getToolkit() {
	return Toolkit.getDefaultToolkit();
    }
    

    public final String getWarningString() {
	return null; // FIXME: implement
    }
    

    public Locale getLocale() {
	return (locale != null) ? locale : Locale.getDefault();
    }
    
    /*
    public InputContext getInputContext()
    */

    /*
    public void setCursor(Cursor cursor) {
	if (cursor == null) cursor = new Cursor(Cursor.DEFAULT_CURSOR);
	super.setCursor(cursor);
    }
    */

    public Window getOwner() {
	return (Window) parent;
    }
    

    public Window[] getOwnedWindows() {
	//Window[] array = new Window[owned.size()];
	//array = owned.toArray(array);
	//return array;
	String msg = "can't be implemented without weak references";
	throw new UnsupportedOperationException(msg);
    }
    
    /*
    public void addWindowListener(WindowListener l);
    public void removeWindowListener(WindowListener l);
    */


    /* this method seem to indicate some kind of unified listener
       collection for the components. */
    /*
    public EventListener[] getListeners(Class listenerType);
    */
 

    protected void processEvent(AWTEvent e) {
	if (e instanceof WindowEvent) {
	    processWindowEvent((WindowEvent) e);
	} else {
	    super.processEvent(e);
	}
    }

    protected void processWindowEvent(WindowEvent e) {
	// FIXME: should post to all window listeners...
    }

    //public Component getFocusOwner();

    public boolean isShowing() {
	return isVisible();
    }

    //public void applyResourceBundle(ResourceBundle rb);
    //public void applyResourceBundle(String rbName);

    //public AccessibleContext getAccessibleContext();

    /** Get graphics configuration. The implementation for
	Window will not ask any parent containers, since Window
	is a toplevel window and not actually embedded in the
	parent component. */
    public GraphicsConfiguration getGraphicsConfiguration() {
	if (peer != null) return peer.getGraphicsConfiguration();
	return null;
    }
}

