package javax.swing.border;

public abstract class AbstractBorder implements Border, java.io.Serializable {

    public AbstractBorder() {};

    public void paintBorder(Component c, Graphics gfx,
			    int x, int y, int width, int height) {
	// default is no painting
    }
    
    /** what the hell is a the BorderMargins the Javadoc mentions? */
    public Insets getBorderInsets(Component c) {
	return new Insets(0, 0, 0, 0);
    }

    public Insets getBorderInsets(Component c, Insets insets) {
	insets.left   = 0;
	insets.right  = 0;
	insets.top    = 0;
	insets.bottom = 0;
	return insets;
    }
    
    // Used by some of the subclasses
    static final Insets INSETS_2_PROTOTYPE = new Insets(2, 2, 2, 2);

    public boolean isBorderOpaque() {
	return false; // by default
    }

    /** Convenience method that maps this to Border argument in the
        method below. */
    public Rectangle getInteriorRectangle(Component c,
					  int x, int y,
					  int width, int height) {
	return getInteriorRectangle(c, this, x, y, width, height);
    }
    

    public static Rectangle getInteriorRectangle(Component c, Border b,
						 int x, int y,
						 int width, int height) {
	Insets insets = b.getBorderInsets(c);
	int l = insets.left;
	int t = insets.top;
	return new Rectangle(x+t, y+t,
			     width-l-insets.right, height-t-insets.bottom);
    }

    /* not part of API, therefore only package accessible.

      Interpretation of the arguments:

       ...................??  . = tlo
       .                   ?  + = tli
       .                   #  % = bri
       .                   #  # = bro
       ?                   #
       ??###################

       corners:           ch
                         |<->|

       ------------------    *  -
                                | cv
                             |  -
                             |

       ......               .....#                .....
            #  ch = 0            #  ch = 1             #   ch = 1
            #                    #                     #
            #  cv = 1            #  cv = 0             #   cv = 1
            #                    #                     #
    */


    static final void paint3DBorder(Graphics gfx,
			       int xl, int yt, int xr, int yb,
			       Color tl, Color br, int ch, int cv) {
	if ((tl == null) || (br == null)) {
	    throw new NullPointerException("null colors not allowed");
	}

	gfx.setColor(tl);
	gfx.drawLine(xl, yb-cv, xl,    yt);
	gfx.drawLine(xl, yt,    xr-ch, yt);

	gfx.setColor(br);
	gfx.drawLine(xr, yt+cv, xr,    yb);
	gfx.drawLine(xr, yb,    xl+ch, yb);
    }
}
