package no.rwr.image;

import java.awt.image.*;
import no.rwr.data.Buffers;

public class ColorConvertOp implements RasterOp {

    private static final boolean DEBUG = false;

    protected ColorModel srcCM;
    protected ColorModel destCM;

    public ColorConvertOp(ColorModel srcCM, ColorModel destCM) {
	this.srcCM  =  srcCM;
	this.destCM = destCM;
    }
    

    public WritableRaster filter(Raster src, WritableRaster dest) {
	if (dest == null) dest = createCompatibleDestRaster(src);

	int x1 = src.getMinX();
	int y1 = src.getMinY();
	int w = src.getWidth();
	int h = src.getHeight();
	int x2 = x1+w;
	int y2 = y1+h;

	if (DEBUG) {
	    System.out.println("performing color convertion");  // DEBUG
	    System.out.println("srcCM:  " + srcCM);
	    System.out.println("destCM: " + destCM);
	}
	Object arrayIn  = null;
	Object arrayOut = null;

	for (int y=y1; y<y2; y++) {
	    if (DEBUG) System.out.println("y=" + y);
	    for (int x=x1; x<x2; x++) {
		int rgb;
		arrayIn = src.getDataElements(x, y, arrayIn);
		rgb = srcCM.getRGB(arrayIn);
		arrayOut = destCM.getDataElements(rgb, arrayOut);
		dest.setDataElements(x, y, arrayOut);
	    }
	}

	return dest;	
    }

    public WritableRaster createCompatibleDestRaster(Raster src) {

	/* FIXME: Maybe we should explicitly create a raster with a
	   tightly pixel packed sample model, rather than assuming
	   that the createCompatibleWritableRaster() method in Raster
	   will create one. */

	return src.createCompatibleWritableRaster();
    }
}
