package no.rwr.image.hermes;

import java.awt.image.*;
import no.rwr.data.Buffers;
import gnu.gcj.RawData;
import no.rwr.image.*;

public class HermesConvertOp implements RasterOp {

    static int getScanlineStride(SampleModel sm) {
	if (sm instanceof ComponentSampleModel) {
	    ComponentSampleModel csm = (ComponentSampleModel) sm;
	    int stride = csm.getScanlineStride();
	    return stride;

	}
	return ((SinglePixelPackedSampleModel) sm).getScanlineStride();
    }

    public HermesConvertOp() {
	init();
    }
    
    public native void init();

    public native void setFormats(HermesFormat src, HermesFormat dest);

    public WritableRaster filter(Raster src, WritableRaster dest) {
	if (dest == null) dest = createCompatibleDestRaster(src);

	DataBuffer srcDB  =  src.getDataBuffer();
	DataBuffer destDB = dest.getDataBuffer();
	
	SampleModel srcSM = src.getSampleModel();
	SampleModel destSM = dest.getSampleModel();


	int  srcScanlineStride = getScanlineStride(srcSM);
	int destScanlineStride = getScanlineStride(destSM);

	int  srcX =  src.getMinX() -  src.getSampleModelTranslateX();
	int  srcY =  src.getMinY() -  src.getSampleModelTranslateY();
	int destX = dest.getMinX() - dest.getSampleModelTranslateX();
	int destY = dest.getMinY() - dest.getSampleModelTranslateY();
	int width = src.getWidth();
	int height = src.getHeight();


	// Convert stride to bytes
	srcScanlineStride *= 
	    DataBuffer.getDataTypeSize(srcSM.getDataType())/8;
	destScanlineStride *= 
	    DataBuffer.getDataTypeSize(destSM.getDataType())/8;

	Object srcData = Buffers.getData(srcDB);
	Object destData = Buffers.getData(destDB);
	

	render(srcData, srcX, srcY, width, height, srcScanlineStride,
	       destData, destX, destY, width, height, destScanlineStride);
	return dest;
    }

    native void render(Object src, int srcX, int srcY,
		       int srcW, int srcH, int srcPitch,
		       Object dest, int destX, int destY,
		       int destW, int destH, int destPitch);

    public WritableRaster createCompatibleDestRaster(Raster src) {

	/* FIXME: Maybe attempt to try to create a more optimized
	   sample model? */
	return src.createCompatibleWritableRaster();
    }

    int handle;
}
