#include <gcj/cni.h>
#include <gnu/gcj/RawData.h>

#include <java/lang/OutOfMemoryError.h>
#include <java/lang/RuntimeException.h>
#include <java/lang/UnsupportedOperationException.h>
#include "no/rwr/image/hermes/HermesFormat.h"
#include "no/rwr/image/hermes/HermesConvertOp.h"
#include "no/rwr/image/hermes/HermesPalette.h"


#include <Hermes/Hermes.h>
#include <iostream>

void no::rwr::image::hermes::HermesConvertOp::init() {
    handle = Hermes_ConverterInstance(0);
    if (handle == 0) {
	jstring msg = JvNewStringLatin1("converter creation failed");
	JvThrow(new java::lang::RuntimeException(msg));
    }
}

void no::rwr::image::hermes::HermesConvertOp::
    setFormats(no::rwr::image::hermes::HermesFormat* src,
	       no::rwr::image::hermes::HermesFormat* dest) {

    typedef ::HermesFormat* HFmt;
    HFmt  srcFmt = reinterpret_cast<HFmt>( src->structure);
    HFmt destFmt = reinterpret_cast<HFmt>(dest->structure);

    if(Hermes_ConverterRequest(handle, srcFmt, destFmt) == 0) {
	jstring msg = JvNewStringLatin1("converter not found");
	JvThrow(new java::lang::RuntimeException(msg));
    }

    HermesPalette* srcPal = 0;
    HermesPalette* destPal = 0;

    jintArray rgba;

    rgba = src->getRGBAPalette();
    if (rgba != 0) {
	srcPal = new HermesPalette();
	srcPal->set(rgba);
    }

    rgba = dest->getRGBAPalette();
    if (rgba != 0) {
	destPal = new HermesPalette();
	destPal->set(rgba);
    }


    if (destPal != 0) {
	char msg[] = 
	    "Cannot convert *to* indexed format. At the time of writing, "
	    "Hermes only support converting *from* indexed palette format. "
	    "The Hermes API allows this to be changed in future. If Hermes "
	    "has begun supporting converting to indexed format, the code "
	    "generating this exception can simply be removed.";
	jstring strMsg = JvNewStringLatin1(msg);
	JvThrow(new java::lang::UnsupportedOperationException(strMsg));
    }

    if ((srcPal != 0) || (destPal != 0)) {
	Hermes_ConverterPalette(handle, srcPal->handle, destPal->handle);
    }
}
	       
void no::rwr::image::hermes::HermesConvertOp::
render(jobject src, jint srcX, jint srcY,
       jint srcW, jint srcH, jint srcPitch,
       jobject dest, jint destX, jint destY,
       jint destW, jint destH, jint destPitch) {
    
    jbyteArray  srcArr = reinterpret_cast<jbyteArray>(src);
    jbyteArray destArr = reinterpret_cast<jbyteArray>(dest);
    void*  srcPtr = reinterpret_cast<void*>(elements(srcArr ));
    void* destPtr = reinterpret_cast<void*>(elements(destArr));
    

    int status =
	Hermes_ConverterCopy(handle,
			      srcPtr,  srcX,  srcY,  srcW,  srcH,  srcPitch,
			     destPtr, destX, destY, destW, destH, destPitch);
    

}
