
package no.rwr.xutil;

import no.rwr.x.*;
import no.rwr.util.BitMaskExtent;

/**
   XID == CARD32 == int == jint



 */

public class ColorContext {
    byte rShift;
    byte gShift;
    byte bShift;
    byte rnBits;
    byte gnBits;
    byte bnBits;
    int rRange;
    int gRange;
    int bRange;
    Visual visual;
    
    /* 
       Pay close attention to bits_per_pixel.
       If bits per pixel is not divisable by 8, then special magic is needed.

       Rather, bits_per_pixel determines what type of magic is needed.
    */




    public static final int VALUE_RANGE = 0x10000;


    public ColorContext(Display display) {
	this(display.getDefaultScreen());
    }

    public ColorContext(Screen screen) {
	// Best visual hunting can be added here.
	this(screen.getRootVisual());
    }

    public ColorContext(Visual visual) {
	this.visual = visual;

	// By default use root window visual

	// *FIXME*: optimize
	long rMask = visual.getRedMask();
	long gMask = visual.getGreenMask();
	long bMask = visual.getBlueMask();
	
	/** If we don't trap empty subfields here, the loops below may
	    never terminate. */
	if ((rMask == 0) || (gMask == 0) || (bMask == 0)) {
	    String errorMessage =
		"Empty RGB subfield masks encountered. " +
		"Maybe the visual (" + visual + ") does not use subfields.";
	    throw new
		RuntimeException(errorMessage);
	};


	/* The bits in the masks are always contiguous without
	   intersections.  */
	BitMaskExtent extent = new BitMaskExtent();

	extent.setMask(rMask);
	rRange = 1 << extent.bitWidth;
	rShift = extent.leastSignificantBit;

	extent.setMask(gMask);
	gRange = 1 << extent.bitWidth;
	gShift = extent.leastSignificantBit;

	extent.setMask(bMask);
	bRange = 1 << extent.bitWidth;
	bShift = extent.leastSignificantBit;


    }

    public final Visual getVisual() {
	return visual;
    }

    /* FIXME. Does anyone really care about having 16 bits per channel.
       If we limit ourselves to 8 bits per channel, we can do better
       optimizations.

       Can we assume that a pixel is never more than 32 bits, thereby
       only use an int as return value?
    */
    public long getPixelValue(int r, int g, int b) {



	/* R, G, B value scaling. We want our pixel value calculation
	   to be the same as returned by XAllocColor(). The following
	   range scaling seems to achieve this. Note that we are working
	   on ranges and not max values. */
	
	/* Under some cirmumstances, this might be reduced further, to
	   simple shifts. When combined with subfield packing this can
	   be optimized even futher if we concentrate on specific
	   packing patterns. */

	/* Alternatively, scaling and subfield shifting can be combined
	   to create lookup tables for RGB values. */

	// It is probably not worth it, except for image convertions.

	/* Possible optimization: Scaling is often just a matter of
	   discarding the lower bits of the input value. This can be
	   achieved by bit-shifting.

	   565 16bit:
	   xyz: x=5, y=6, z=5

	   rmask == 1111100000000000 == 0xf800
	   gmask == 0000011111100000 == 0x07e0
	   bmask == 0000000000011111 == 0x001f

	   [rrrrrrrrrrrrrrrr]  [gggggggggggggggg]  [bbbbbbbbbbbbbbbb]
            |||||               ||||||              |||||
	    `````----------.....||||||,,,,,---------'''''
	 		   ||||||||||||||||
                          [rrrrrggggggbbbbb]

	    pixel = (r & rmask) | ((g>>x) & gmask) | ((b>>(x+y)) & bmask)
	    		3 ANDs, 2 ORs, 2 shifts

		alt:

	    pixel = (r & rmask) | ((g>>x) & gmask) | (b>>(x+y))
	    		2 ANDs, 2 ORs, 2 shifts
	    
	    	alt:

	    pixel = ((r>>(y+z))<<(y+z)) | ((g>>(x+z))<<z) | (b>>(x+y))
	                2 ORs, 5 shifts

	        alt:

	    pixel = (((b>>y) | (g & gmask))>>x) | (r & rmask) 
	                2 ANDs, 2 ORs, 2 shifts    

	        or variations thereof. See what generates most
	        efficient code. Some compilers might do clever things
	        with the shift expressions.


	    However, the RGB values are usually prepacked:
	                          |<-- 11 ->|
                           |<--- 13 -->|
                  |<---- 16 ---->|
             ____    ____    ____    ____    
            [rrrrrrrrggggggggbbbbbbbb????????] int
	     |||||   ||||||  `````------.....
             |||||   ``````-------......|||||
             `````-----------.....|||||||||||
	                    [rrrrrggggggbbbbb] short
            [rrrrrrrrggggggggbbbbbbbb????????] int
			    

                             ----    ----
	    pixel = ((rgb>>16) & rmask) |
	            ((rgb>>13) & gmask) |
	            ((rgb>>11) & bmask);
                3 ANDs, 2 ORs, 3 shifts

	    alt:

	    pixel = ((rgb>>27)<<11)     |
	            ((rgb>>13) & gmask) |
		    ((rgb<<16)>>27)

	    [????????rrrrrrrrggggggggbbbbbbbb] int
                      \\\\\  ||||||  ||||| 
                       \\\\\ ||||||  |||||  
                        \\\\\ \\\\\\ |||||   
                         \\\\\ \\\\\\ \\\\\  
                          \\\\\ \\\\\\ \\\\\
                           \\\\\ \\\\\\ |||||
                            \\\\\ |||||||||||
                            [rrrrrggggggbbbbb] short
	             

	*/

	int rv = r*rRange/VALUE_RANGE;
	int gv = g*gRange/VALUE_RANGE;
	int bv = b*bRange/VALUE_RANGE;

	// r, g, b subfield packing
	long pixel = (rv<<rShift) | (gv<<gShift) | (bv<<bShift);

	return pixel;
    }




    /* A color context is only valid for a specific screen, since a
       color context may bound to a specific color map and a color map
       is only valid within a specific screen. */

    // Screen screen;
    // long colormapXID;

    //    ColorContext(
    //		 /* Display display, */
    //		 colormapXID colormap
    //		 );


    /* If we use SHM, can we assume that the server and client is
       using the same endianess?
    */


    /*
      Digging information out of X structures:

      display->default_screen
          screen->default_colormapXID
	      

	  screen->depths (Depth*, nDepth)
	      Depth->visual
	          Visual->class
		        ->bits_per_rgb
		        ->id
			For classes TRUECOLOR and DIRECTCOLOR:
			->RGB pixel masks
	  screen->
	          width (pixels, millimeter)
		  height (pixels, millimeter)
		  root
		  root_depth
		  root_visual

      display->XGetVisualInfo()->xvisualinfos
          XVisualInfo->...

      {depth, visual, [[colormap]]} -> XCreateWindow()->drawable
          drawable->GC

      For PsuedoColor and StaticColor:
          - Allocate colors

      colormap->?->screen

      pixel-value == a field of bits denoting a color

      visual      == color to pixel-value conversion

      colormap    == serverside translation of pixel values to display values
                     A colormap is NOT (?) visual specific

      colormap is not interresting at all in TRUE mode

      Pixmap == server side raster imagery
      
      XImage == client memory image data managemnt stucture. Is not the
                owner of the data. Simply a description of existing
		memory allocation.

      // An XImage can be drawn into a Pixmap
      XImage -> XPutImage() -> Pixmap

      // If an image is likely to be drawn more than once, move the
      // image to a Pixmap before painting.

     */

    // A GC is not bound to a drawable


}
